inlets = 2;
outlets = 2;

// Global Constants
var CONTROL_SURFACE = "control_surfaces 0 ";

// Listener Arrays
var buttonShift;
var buttonArm = [];
var buttonCrossfaderSelect = [];
var buttonSolo = [];
var buttonMute = [];
var buttonCL = [];
var buttonMFX = [];
var buttonLoopToggle = [];
var buttonTrackActivate = [];
var buttonFXReset = [];
var clipLaunchListener = [];

// Global Dynamics
var boolEnabled = 0;				// Checked by the init() function to determine if the template setup has taken place or not. Prevents duplicate instances.
var boolCueToggle = 0;

// Random usage variable
var lAPI;
var i;
var j;
var intArray;


function bang() {
	if (inlet == 0 && !boolEnabled) {
		init();
	}
}

// function Stop()
// Kills the entire setup. Probably gonna be a bunch of work. To be determined if it's even necessary at this point.

function Stop() {
	
}

// function anything()
// Catches any weirdness from the patcher being sent to this script.

function anything() {
	post("Anything Received: " + arrayfromargs(arguments) + " on Inlet #" + inlet + "\n");
}

// function path()
// Listens for the path argument from the m4l.api.selectcontrolsurface patcher
// that allows the user to pull their specific control surface. This kicks off 
// the show by making sure the name matches "APC40_MkII" and running the init()
// function

function path() {
	switch (inlet) {
		case 0:
			CONTROL_SURFACE = arrayfromargs(arguments)[0] + " " + arrayfromargs(arguments)[1] + " ";
			post("Control Surface Selected.\n");
			break;
		default:
			break;
	}
}


function init() {

	lAPI = new LiveAPI();
	
	 LightShow();					// Light show for startup sequence.

	//disableSLButtons();			// Broken on APC40 MKii. I haven't figured out how to do this without severely breaking the controller. No worries.
	
	disableRightGrid();
	disableTrackSelectButtons();
	SetupLoopControlButtons();
	SetupSendControlButtons();
	setupCLGrid();
	setupMFXGrid();
	setupLoopToggleButtons();
	setupEffectResetButtons();
	disableDeviceControlButtons();
	SetupShiftButton();
	outlet(0, "bang");				// Sets the parameter values if being loaded fresh with presets
	outlet(1, "outputvalue");		// Sets the state of cue toggling. This is important to users with only one set out outputs on their soundcard (avoids solo situations)
	boolEnabled = 1;				// Ensures no duplicate instances are spawned.

	
	post("Initialized.\n");
}

function LightShow() {
	for(i = 127; i >= 0; i --) {
		for(j = 22; j <= 61; j ++) {
			lAPI.path = CONTROL_SURFACE + "controls " + j;
			lAPI.call("send_value", i);
		}
	}

	lAPI.id = 0;	
}

// Shift Button
// The following code establishes the shift button listener. Default functionality is intact, however this just resets the lights
// upon coming back from a shift press.

function SetupShiftButton() {
	buttonShift = new LiveAPI(ShiftCallback, "control_surfaces 0 controls 0");
	buttonShift.property = "value";
	
	post("Shift Button Listener Setup.\n");
}

function ShiftCallback(args) {
	if (args[0] == "value") {
	
		switch(args[1]) {	
			case 0:
				post("Shift Button Released.\n");
				break;
			case 127:
				post("Shift Button Pressed.\n");
				break;
			default:
				break;
		}	
	}	
}

// TRACK SELECT BUTTONS
// The following disable/setup function and callback handle the track select buttons. It allows the user to detail/cue the playing clip
// on any given track.
// 

function disableTrackSelectButtons() {
	// Setup track select buttons to bring up the playing clip to detail view.
	for(i = 0; i <= 3; i ++) {
		buttonTrackActivate[i] = new LiveAPI(ClipViewCallback, CONTROL_SURFACE + "controls " + (95 + i));
		buttonTrackActivate[i].TrackID = i;
		buttonTrackActivate[i].property = "value";
	}
	
}

function ClipViewCallback(args) {
	switch(args[1]) {
		case 127:
			SetDetailCue(this.TrackID);
			break;
		default:
			break;
	}
	
	lAPI.id = 0;
}


// DETAIL/CUE FUNCTION
// Used by the preceding two function sets. This little dandy will navigate to a track, find the current
// playing clip, and then focus it in detail view and solo/cue the track. I use this for setting up my decks ahead of time with
// my headphones.
//

function SetDetailCue(intTrackID) {
	lAPI.path = "live_set tracks " + intTrackID;
	if (lAPI.get("playing_slot_index") >= 0) {
		lAPI.path = "live_set tracks " + intTrackID + " clip_slots " + lAPI.get("playing_slot_index") + " clip";
		i = lAPI.id;
		lAPI.path = "live_set view";
		lAPI.set("detail_clip", "id", i);
		lAPI.path = "live_app view";
		lAPI.call("show_view", "Detail/Clip");
		SetCue(intTrackID);
	}
}

// SetCue(args)
// Sets the cue state of any given track.

function SetCue(args) {
	if (boolCueToggle == 1) {
		for (i = 0; i <= 3; i++) {
			lAPI.path = "live_set tracks " + i;
			if (i != args) {
				lAPI.set("solo", 0);
			} else {
				lAPI.set("solo", 1);
			}
		}
	}
}

// CueToggle()
// Listens for CueToggle messages and Sets cue toggle state accordingly.

function CueToggle() {
	switch (inlet) {
		case 1:
			boolCueToggle = arrayfromargs(arguments);
			break;
		default:
			break;
	}
}

// Send Buttons.

function SetupSendControlButtons() {
	// Disables the Activator, Solo/Cue, and Record Arm buttons
	intArray = [64, 127, 136, 118, 109];
	
	// Tracks 6 & 7 Setup and Assignment of Send Buttons and Callback Information
	
	for(i = 0; i <= 1; i ++) {
		lAPI.path = CONTROL_SURFACE + "components " + (intArray[0] + i);
		lAPI.call("set_solo_button", "id", 0);
		lAPI.call("set_crossfade_toggle", "id", 0);
		lAPI.call("set_mute_button", "id", 0);
		lAPI.call("set_arm_button", "id", 0);
		
		// Track Activator 		for Track 1
		buttonMute[i + 6] = new LiveAPI(SendCallback, CONTROL_SURFACE + "controls " + (intArray[1] + i));
		buttonMute[i + 6].TrackID = 0;
		buttonMute[i + 6].Send = i;
		buttonMute[i + 6].property = "value";

		// Crossfader Selector 	for Track 2
		buttonCrossfaderSelect[i + 6] = new LiveAPI(SendCallback, CONTROL_SURFACE + "controls " + (intArray[2] + i));
		buttonCrossfaderSelect[i + 6].TrackID = 1;
		buttonCrossfaderSelect[i + 6].Send = i;
		buttonCrossfaderSelect[i + 6].property = "value";

		// Solo / Cue 			for Track 3
		buttonSolo[i + 6] = new LiveAPI(SendCallback, CONTROL_SURFACE + "controls " + (intArray[3] + i));
		buttonSolo[i + 6].TrackID = 2;
		buttonSolo[i + 6].Send = i;
		buttonSolo[i + 6].property = "value";

		// Record / Arm 		for Track 4
		buttonArm[i + 6] = new LiveAPI(SendCallback, CONTROL_SURFACE + "controls " + (intArray[4] + i));
		buttonArm[i + 6].TrackID = 3;
		buttonArm[i + 6].Send = i;
		buttonArm[i + 6].property = "value";

	}
	
	lAPI.id = 0;
	post("Send Control Buttons (Crossfade, Solo/Cue, Activate, Record) Disabled.\n");
	post("Send Buttons Initialized.\n");
}

function SendCallback(args) {
	
	if (args[0] == "value") { 	// This line is a little extraneous, but should probably _really _ be added to all similar functions. Callback is  
								// triggered upon variable instantiation with "id" and the actual id #. If for some reason the id was 127, it 
								// would call. Here we make sure that it is a "value" and not an "id" being passed.
		switch (args[1]) {
	 		case 127:
				post("Send enabled on Track " + (this.TrackID + 1) + " for Send " + String.fromCharCode(65 + this.Send) + ".\n");
				lAPI.path = "live_set tracks " + this.TrackID + " mixer_device sends " + this.Send;
				if (lAPI.get("value") < 1) {
					this.call("send_value", 1);
					lAPI.set("value", 1);
				} else {
					lAPI.set("value", 0);
					this.call("send_value", 0);
				}
				
				break;
			default:
				break;
		}
	}

	lAPI.id = 0;

}



// Looping buttons. Take control of the track activator, solo/cue, crossfade, and record
// buttons to turn them into loop buttons. 
//


function SetupLoopControlButtons() {
	// For funsies, we disable the Stop_All_Clips_Button here, too.
	lAPI.path = CONTROL_SURFACE + "components 1";
	lAPI.call("set_stop_all_clips_button", "id", 0);

	// Disables the Activator, Solo/Cue, and Record Arm buttons
	intArray = [58, 121, 130, 112, 103];
	
	// Tracks 1 through 4 Setup and Assignment of Looping Buttons and Callback Information
	
	for(i = 0; i <= 3; i ++) {
		lAPI.path = CONTROL_SURFACE + "components " + (intArray[0] + i);
		lAPI.call("set_solo_button", "id", 0);
		lAPI.call("set_crossfade_toggle", "id", 0);
		lAPI.call("set_mute_button", "id", 0);
		lAPI.call("set_arm_button", "id", 0);
		
		// Track Activator 		at 4 beats
		buttonMute[i] = new LiveAPI(LoopCallback, CONTROL_SURFACE + "controls " + (intArray[1] + i));
		buttonMute[i].TrackID = i;
		buttonMute[i].Beats = 4;
		buttonMute[i].property = "value";

		// Crossfader Selector 	at 8 beats
		buttonCrossfaderSelect[i] = new LiveAPI(LoopCallback, CONTROL_SURFACE + "controls " + (intArray[2] + i));
		buttonCrossfaderSelect[i].TrackID = i;
		buttonCrossfaderSelect[i].Beats = 8;
		buttonCrossfaderSelect[i].property = "value";

		// Solo / Cue 			at 16 beats
		buttonSolo[i] = new LiveAPI(LoopCallback, CONTROL_SURFACE + "controls " + (intArray[3] + i));
		buttonSolo[i].TrackID = i;
		buttonSolo[i].Beats = 16;
		buttonSolo[i].property = "value";

		// Record / Arm 		at 32 beats
		buttonArm[i] = new LiveAPI(LoopCallback, CONTROL_SURFACE + "controls " + (intArray[4] + i));
		buttonArm[i].TrackID = i;
		buttonArm[i].Beats = 32;
		buttonArm[i].property = "value";

	}
	
	lAPI.id = 0;
	post("Track Control Buttons (Crossfade, Solo/Cue, Activate, Record) Disabled.\n");
	post("Loop Buttons Initialized.\n");
}



function LoopCallback(args) {
	var intPP;
	
	switch (args[1]) {
		case 127:
			lAPI.path = "live_set tracks " + this.TrackID;
			if (lAPI.get("playing_slot_index") >= 0) {
				lAPI.path = "live_set tracks " + this.TrackID + " clip_slots " + lAPI.get("playing_slot_index") + " clip";
				intPP = parseInt(lAPI.get("playing_position"), 10);
				lAPI.set("looping", 1);
				if (lAPI.get("loop_start") > intPP) {
					lAPI.set("loop_start", intPP);
					lAPI.set("loop_end", intPP + this.Beats); 
				} else {
					lAPI.set("loop_end", intPP + this.Beats); 
					lAPI.set("loop_start", intPP);
				}
				buttonLoopStop[this.TrackID].call("send_value", 45);
			}
			lAPI.id = 0;
			break;
		default:
			break;
	}
}	

// The following functions disable the Device Control Buttons (bottom eight buttons on APC40)
// We use the Device <- / Device ->, Bank <- / Bank -> buttons as effect resets for each respective
// deck #'s 1-4
//

function disableDeviceControlButtons() {
	// Finally let's light them up all pretty lookin!
	for(i = 184; i <= 191; i++) {
		lAPI.path = CONTROL_SURFACE + "controls " + i;
		lAPI.call("turn_on");
	}
	// Disable the "Dev On/Off" and "Device Lock" Buttons
	lAPI.path = CONTROL_SURFACE + "components 70";
	lAPI.call("set_enabled", 0);

	// The next two lines disable the "Clip/Track", "Detail View", "<-", and "->" buttons
	lAPI.path = CONTROL_SURFACE + "components 71";
	lAPI.call("set_enabled", 0);

	// Post about it to brag!
	post("Bottom eight buttons disabled and lit up.\n");	
}

function setupEffectResetButtons() {
	for(i = 0; i <= 3; i++) {
		buttonFXReset[i] = new LiveAPI(FXResetCallback, CONTROL_SURFACE + "controls " + (i + 184));
		buttonFXReset[i].n = i;
		buttonFXReset[i].property = "value";
	}
	
	lAPI.id = 0;
	post("Effect reset buttons setup.\n");
	
}

function FXResetCallback(args) {
	switch (args[1]) {
		case 127:
			//post("Reset effects on channel: " + this.n + "\n");
			lAPI.path = "live_set tracks " + this.n + " devices 0 parameters 1";
			lAPI.set("value", 63);
			for (i = 2; i <= 8; i++) {
				lAPI.path = "live_set tracks " + this.n + " devices 0 parameters " + i;
				lAPI.set("value", 0);
			}
			break;
		default:
			break;
	}
	lAPI.id = 0;		
}

// RIGHT GRID FUNCTIONS: DISABLE, SETUP, CALLBACK(s)
// These functions all relate to the right-most 20 buttons on the APC40 Clip Launch Grid
// 

function disableRightGrid() {
	// Disable right 20 clip launch buttons
	for(i = 16; i <= 54; i++) {
		button = new LiveAPI(CONTROL_SURFACE + "components " + i);
		button.call("set_enabled", 0);
		if ((i - 1) % 9 === 0) {
			i = i + 5;
		}
	}

	post("Right Grid Disabled.\n");

}

// FIXED CLIP LAUNCH GRID SETUP AND CALLBACK

function setupCLGrid() {
	intArray = [26, 34, 42, 50];

	for(j = 0; j <= 1; j++) {
		for(i = 0; i <= 3; i ++) {
			buttonCL[i + (j * 3)] = new LiveAPI(ClipLaunchCallback, CONTROL_SURFACE + "controls " + (intArray[i] + j));
			lAPI.path = "live_set tracks " + (4 + j) + " clip_slots " + i;
			buttonCL[i + (j * 3)].ClipLaunchID = lAPI.id;
			buttonCL[i + (j * 3)].ColorOn = 122;
			buttonCL[i + (j * 3)].ColorOff = 95;
			buttonCL[i + (j * 3)].property = "value";
			buttonCL[i + (j * 3)].call("send_value", buttonCL[i + (j * 3)].ColorOff);
		}
		post("Deck " + (j + 1) + " Fixed Clip Launch Buttons Setup.\n");
	}
	
	lAPI.id = 0;	
}

function ClipLaunchCallback(args) {
	switch (args[1]) {		// Operate based on button pressed or not (127 == pressed, 0 == not pressed)
		case 127:
			this.call("send_value", this.ColorOn);
			lAPI.id = parseInt(this.ClipLaunchID, 10);
			lAPI.call("fire");
			break;
		case 0:
			this.call("send_value", this.ColorOff);
			break
		default:
			break;		
	}	

	lAPI.id = 0;
}


// MOMENTARY FX GRID SETUP AND CALLBACK

function setupMFXGrid() {
	intArray = [28, 36, 44, 52];

	for(j = 0; j <= 1; j++) {
		for(i = 0; i <= 3; i ++) {
			buttonMFX[i + (j * 4)] = new LiveAPI(MFXCallback, CONTROL_SURFACE + "controls " + (intArray[i] + j));
			buttonMFX[i + (j * 4)].Deck = j;
			buttonMFX[i + (j * 4)].ID = (i + (j * 4));
			buttonMFX[i + (j * 4)].LOMID = 0;
			buttonMFX[i + (j * 4)].ParameterValue = 0;
			buttonMFX[i + (j * 4)].ColorOn = 9;
			buttonMFX[i + (j * 4)].ColorOff = 89;
			buttonMFX[i + (j * 4)].property = "value";
			buttonMFX[i + (j * 4)].call("send_value", buttonMFX[i + (j * 4)].ColorOff);
		}
		post("MFX Deck " + (j + 1) + " Setup Successfully\n");
	}
	lAPI.id = 0;
}


// function MFX()
// Reads MFX updates from any of the parameters
// and assigns the values accordingly.

function MFX() {
	post("MFX Received! : " + arrayfromargs(arguments) + "\n");
	buttonMFX[arrayfromargs(arguments)[0]].LOMID = arrayfromargs(arguments)[2];
	buttonMFX[arrayfromargs(arguments)[0]].ParameterValue = arrayfromargs(arguments)[3];
}


function MFXCallback(args) {
	if (args[0] == "value") {
		switch (args[1]) {
			case 127:
				this.call("send_value", this.ColorOn);
				post("MFX Column " + this.Deck + " id " + this.ID + " Momentary On.\n");
				lAPI.id = this.LOMID;
				lAPI.set("value", this.ParameterValue);
				break;
			case 0:
				this.call("send_value", this.ColorOff);
				post("MFX Column " + this.Deck + " id " + this.ID + " Momentary Off.\n");
				lAPI.id = this.LOMID;
				lAPI.set("value", 0);
				break;
			default:
				break;
		}	
	}
}



// LOOP TOGGLES
// These function assign the bottom four of the clip launch grid to their respective channel to indicate a loop
// is in effect and allow you to toggle looping for on/off as well.
//

function setupLoopToggleButtons() {	
	// Assign button variables for loop-stoppage in the callback function above.
	for(i = 58; i <= 61; i ++) {
		buttonLoopToggle[i - 58] = new LiveAPI(LoopToggleCallback, CONTROL_SURFACE + "controls " + i);
		buttonLoopToggle[i - 58].TrackID = (i - 58);
		buttonLoopToggle[i - 58].ColorOn = 45;
		buttonLoopToggle[i - 58].ColorOff = 72;
		buttonLoopToggle[i - 58].property = "value";
		buttonLoopToggle[i - 58].call("send_value", 72);
	}
	
	post("Loop Stop Buttons Setup.\n");
	
}

function LoopToggleCallback(args) {
	switch (args[1]) {
		case 127:
			lAPI.path = "live_set tracks " + this.TrackID;
			if (lAPI.get("playing_slot_index") >= 0) {
				lAPI.path = "live_set tracks " + this.TrackID + " clip_slots " + lAPI.get("playing_slot_index") + " clip";
				if (lAPI.get("looping") == 1) {	
					lAPI.set("looping", 0);
					buttonLoopStop[this.TrackID].call("send_value", this.ColorOff);
				} else {
					lAPI.set("looping", 1);
					buttonLoopStop[this.TrackID].call("send_value", this.ColorOn);
				}
			}
			lAPI.id = 0;
			break;
		default:
			break;
	}
	
}


// Not working with the APC40 MKII - investigate and fix or leave behind?
//

function disableSLButtons() {
	// Disable Scene Launch Buttons
		
	for(i = 11; i <= 47; i += 9) {
		
		post("Disabling component #" + i + ".\n");
		lAPI.path = CONTROL_SURFACE + "components " + i;
		lAPI.call("set_launch_button", "id", 0);
	}
	
	for(i = 16; i <=20; i++) {
		lAPI.path = CONTROL_SURFACE + "controls " + i;
		lAPI.call("send_value", 44);
	}
	
	lAPI.id = 0;
	post("Scene Launce buttons disabled and lit up.\n");
}



